using System;
using System.Text;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;

namespace Lexer
{

    public class LexerException : System.Exception
    {
        public LexerException(string msg)
            : base(msg)
        {
        }

    }

    public class Lexer
    {

        protected int position;
        protected char currentCh; //   
        protected int currentCharValue; //     
        protected System.IO.StringReader inputReader;
        protected string inputString;

        public Lexer(string input)
        {
            inputReader = new System.IO.StringReader(input);
            inputString = input;
        }

        public void Error()
        {
            System.Text.StringBuilder o = new System.Text.StringBuilder();
            o.Append(inputString + '\n');
            o.Append(new System.String(' ', position - 1) + "^\n");
            o.AppendFormat("Error in symbol {0}", currentCh);
            throw new LexerException(o.ToString());
        }

        protected void NextCh()
        {
            this.currentCharValue = this.inputReader.Read();
            this.currentCh = (char) currentCharValue;
            this.position += 1;
        }

        public virtual bool Parse()
        {
            return true;
        }
    }

    public class IntLexer : Lexer
    {

        protected System.Text.StringBuilder intString;
        public int parseResult = 0;

        public IntLexer(string input)
            : base(input)
        {
            intString = new System.Text.StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
			int symb = 1;
            if (currentCh == '+' || currentCh == '-')
            {
				if (currentCh == '-')
					symb *= -1;
                NextCh();
            }
			int res = 1;
        
            if (char.IsDigit(currentCh))
            {
				res *= currentCh - '0';

				NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh))
            {
				res = res*10 + currentCh - '0';
				NextCh();
            }


            if (currentCharValue != -1)
            {
                Error();
            }

			parseResult = symb * res;

            return true;

        }
    }
    
    public class IdentLexer : Lexer
    {
        private string parseResult;
        protected StringBuilder builder;
    
        public string ParseResult
        {
            get { return parseResult; }
        }
    
        public IdentLexer(string input) : base(input)
        {
            builder = new StringBuilder();
        }

		public override bool Parse()
		{
			// throw new NotImplementedException();
			NextCh();

			if (currentCharValue == -1)
			{
				Error();
			}

			var bld = new StringBuilder();
			if (char.IsLetter(currentCh) || currentCh == '_')
			{
				bld.Append(currentCh);
				NextCh();
			}
			else
				return false;
			while (char.IsLetter(currentCh) || char.IsDigit(currentCh) || currentCh == '_')
			{
				bld.Append(currentCh);
				NextCh();
			}

			if (currentCharValue != -1)
			{
				Error();
			}

			parseResult = bld.ToString();

			return true;

		}
       
    }

    public class IntNoZeroLexer : IntLexer
    {
        public IntNoZeroLexer(string input)
            : base(input)
        {
        }

        public override bool Parse()
        {
			//throw new NotImplementedException();
			NextCh();

			if (currentCharValue == -1 || currentCh == '0')
			{
				Error();
			}

			if (currentCharValue == '-' || currentCh == '+')
			{
				NextCh();
				if (currentCharValue == -1 || currentCh == '0')
				{
					Error();
				}
			}
			NextCh();

			while (char.IsDigit(currentCh))
			{
				NextCh();
			}

			if (currentCharValue != -1)
			{
				Error();
			}

			return true;
		}
    }

    public class LetterDigitLexer : Lexer
    {
        protected StringBuilder builder;
        protected string parseResult;

        public string ParseResult
        {
            get { return parseResult; }
        }

        public LetterDigitLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
			//throw new NotImplementedException();
			NextCh();
			if (currentCharValue == -1)
			{
				Error();
			}

			if (char.IsDigit(currentCh))
			{
				Error();
			}
			else if (char.IsLetter(currentCh))
			{
				while (char.IsLetter(currentCh) || char.IsDigit(currentCh))
				{
					NextCh();
					if (char.IsLetter(currentCh))
					{
						Error();
					}
					NextCh();
					if (char.IsDigit(currentCh))
					{
						Error();
					}
				}
				
			}

			if (currentCharValue != -1)
			{
				Error();
			}

			return true;
		}
       
    }

    public class LetterListLexer : Lexer
    {
        protected List<char> parseResult;

        public List<char> ParseResult
        {
            get { return parseResult; }
        }

        public LetterListLexer(string input)
            : base(input)
        {
            parseResult = new List<char>();
        }

        public override bool Parse()
        {
            //throw new NotImplementedException();
            NextCh();
            if (currentCharValue == -1)
            {
                Error();
            }

            if (currentCh == ',' || currentCh == ';' || char.IsDigit(currentCh))
            {
                Error();
            }

            bool fl = true; //  - 

            while (char.IsLetter(currentCh) || currentCh == ',' || currentCh == ';')
            {
                if ((currentCh == ',' || currentCh == ';') && fl == false)
                {
                    fl = true;
                    NextCh();
                }
                else 
                if (char.IsLetter(currentCh) && fl)
                {
                    ParseResult.Add(currentCh);
                    fl = false;
                    NextCh();
                } else Error();
            }

            if (currentCharValue != -1 || fl)
            {
                Error();
            }

            return true;
        }
    }

    public class DigitListLexer : Lexer
    {
        protected List<int> parseResult;

        public List<int> ParseResult
        {
            get { return parseResult; }
        }

        public DigitListLexer(string input)
            : base(input)
        {
            parseResult = new List<int>();
        }

        public override bool Parse()
        {
            //throw new NotImplementedException();
            NextCh();
            if (currentCharValue == -1 || currentCh == ' ')
            {
                Error();
            }

            bool fl = true;
            while (currentCh == ' ' || char.IsDigit(currentCh))
            {
                if (currentCh == ' ')
                {
                    fl = true;
                }
                else
                {
                    if (char.IsDigit(currentCh) && fl)
                    {
                        fl = false;
                        parseResult.Add(currentCh - '0');
                       
                    }
                    else Error();
                }

                NextCh();
            }

            if (currentCharValue != -1 || fl)
            {
                Error();
            }

            return true;
        
        }
    }

    public class LetterDigitGroupLexer : Lexer
    {
        protected StringBuilder builder;
        protected string parseResult;

        public string ParseResult
        {
            get { return parseResult; }
        }
        
        public LetterDigitGroupLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            //throw new NotImplementedException();
            NextCh();
            if (currentCharValue == -1 || !char.IsLetter(currentCh) )
            {
                Error();
            }

            int fl1 = 0;// . 0 -  , 1 - , 2 - 2 
            int fl2 = 0;
            while (char.IsLetter(currentCh) || char.IsDigit(currentCh))
            {
                if (char.IsLetter(currentCh))
                {
                    fl2 = 0;
                    if (fl1 == 2)
                    {
                       Error();
                    }
                    if (fl1 == 1)
                    {
                        fl1 = 2;
                        builder.Append(currentCh);
                    }
                    if (fl1==0)
                    {
                        fl1 = 1;
                        builder.Append(currentCh);
                    }
                }
                else
                {
                    fl1 = 0;
                    if (fl2 == 2)
                    {
                        Error();
                    }
                    if (fl2 == 1)
                    {
                        fl2 = 2;
                        builder.Append(currentCh);
                    }
                    if (fl2 == 0)
                    {
                        fl2 = 1;
                        builder.Append(currentCh);
                    }
                }
               

                NextCh();
            }

            if (currentCharValue != -1)
            {
                Error();
            }
            parseResult = builder.ToString();
            return true;
        }
       
    }

    public class DoubleLexer : Lexer
    {
        private StringBuilder builder;
        private double parseResult;

        public double ParseResult
        {
            get { return parseResult; }

        }

        public DoubleLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            //throw new NotImplementedException();
            NextCh();
            if (currentCharValue == -1 || currentCh == '.')
            {
                Error();
            }

            bool flag = false; //  
            //bool ef = false;
            double mult = 1;
            while (currentCh == '.' || char.IsDigit(currentCh))
            {
                if (char.IsDigit(currentCh) && flag)
                {
                    //ef = true;
                    mult /= 10;
                    parseResult = parseResult + (currentCh - '0') * mult;
                    NextCh();
                }
                if (char.IsDigit(currentCh) && !flag)
                {
                    parseResult = parseResult * 10 + (currentCh - '0');
                    NextCh();
                }
                if (currentCh == '.')
                {
                    if (flag)
                        Error();
                    flag = true;
                    NextCh();
                    if (!char.IsDigit(currentCh))
                        Error();
                }


                //NextCh();
            }

            //if (!flag && ef)
            //    Error();

            if (currentCharValue != -1)
            {
                Error();
            }

            return true;
        }
       
    }

    public class StringLexer : Lexer
    {
        private StringBuilder builder;
        private string parseResult;

        public string ParseResult
        {
            get { return parseResult; }

        }

        public StringLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            //throw new NotImplementedException();

            NextCh();
            if (currentCharValue == -1 || currentCh != '\'')
            {
                Error();
            }

            bool fl = false;
            while (currentCharValue != -1)
            {
                builder.Append(currentCh);
                NextCh();
                if (currentCharValue != -1 && fl)
                    Error();
                if (currentCh == '\'')
                {
                    fl = true;
                }

            }

            if (currentCharValue != -1 || !fl)
            {
                Error();
            }
            parseResult = builder.ToString();
            return true;
        }
    }

    public class CommentLexer : Lexer
    {
        private StringBuilder builder;
        private string parseResult;

        public string ParseResult
        {
            get { return parseResult; }

        }

        public CommentLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            //throw new NotImplementedException();

            NextCh();
            if (currentCharValue == -1 || currentCh != (char)(47))
            {
                Error();
            }
            
            NextCh();
            if (currentCharValue == -1 || currentCh != '*')
            {
                Error();
            }
            NextCh();




            int fl = 0; // 0 - , 1 - * , 2 - 
            while (currentCharValue != -1)
            {
                if (currentCh == (char)(47) && fl == 1)
                {
                    fl = 2;
                    NextCh();
                    if (currentCharValue != -1)
                    {
                        Error();
                    }
                    return true;
                }
                if (currentCh == '*')
                {
                    fl = 1;
                }
                else fl = 0;
                //builder.Append(currentCh);
                NextCh();
            }

            if (currentCharValue != -1 || fl != 2)
            {
                Error();
            }
            //parseResult = builder.ToString();
            return true;
        }
    }

    public class IdentChainLexer : Lexer
    {
        private StringBuilder builder;
        private List<string> parseResult;

        public List<string> ParseResult
        {
            get { return parseResult; }

        }

        public IdentChainLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
            parseResult = new List<string>();
        }

        public override bool Parse()
        {
            throw new NotImplementedException();
        }
    }

    public class Program
    {
        public static void Main()
        {
            string input = "154216";
            Lexer L = new IntLexer(input);
            try
            {
                L.Parse();
            }
            catch (LexerException e)
            {
                System.Console.WriteLine(e.Message);
            }

        }
    }
}