%{
//      GPPGParser,   ,   gppg
    public BlockNode root; //     
    public Parser(AbstractScanner<ValueType, LexLocation> scanner) : base(scanner) { }
%}

%output = SimpleYacc.cs

%union { 
			public double dVal; 
			public int iVal; 
			public string sVal; 
			public Node nVal;
			public ExprNode eVal;
			public StatementNode stVal;
			public BlockNode blVal;
			public VarDefNode vdVal;
       }

%using ProgramTree;

%namespace SimpleParser

%nonassoc IFX
%nonassoc ELSE

%token BEGIN END CYCLE ASSIGN SEMICOLON WHILE DO REPEAT UNTIL FOR TO WRITE LEFT_PR RIGHT_PR IF THEN VAR COMMA PLUS MINUS MULT DIVISION
%token <iVal> INUM 
%token <dVal> RNUM 
%token <sVal> ID

%type <eVal> expr ident t f
%type <stVal> assign statement cycle while repeat for write if var
%type <blVal> stlist block
%type <vdVal> idList

%%

progr   : block { root = $1; }
		;

stlist	: statement 
			{ 
				$$ = new BlockNode($1); 
			}
		| stlist SEMICOLON statement 
			{ 
				$1.Add($3); 
				$$ = $1; 
			}
		;

statement: assign { $$ = $1; }
		| block   { $$ = $1; }
		| cycle   { $$ = $1; }
		| while   { $$ = $1; }
		| repeat  { $$ = $1; }
		| for     { $$ = $1; }
		| write   { $$ = $1; }
		| if      { $$ = $1; }
		| var     { $$ = $1; }
	;

ident 	: ID { $$ = new IdNode($1); }	
		;
	
assign 	: ident ASSIGN expr { $$ = new AssignNode($1 as IdNode, $3); }
		;

expr    : t { $$ = $1; }
		| t PLUS expr { $$ = new BinaryNode($1, $3, BinOpType.Plus); }
		| t MINUS expr { $$ = new BinaryNode($1, $3, BinOpType.Minus); }
		;

t       : f { $$ = $1; }
		| t MULT f { $$ = new BinaryNode($1, $3, BinOpType.Mult); }
		| t DIVISION f { $$ = new BinaryNode($1, $3, BinOpType.Division); }
		;


f       : ident { $$ = $1 as IdNode; }
		| INUM { $$ = new IntNumNode($1); }
		| LEFT_PR expr RIGHT_PR { $$ = $2; }
		;


block	: BEGIN stlist END { $$ = $2; }
		;

cycle	: CYCLE expr statement { $$ = new CycleNode($2, $3); }
		;
	
while   : WHILE expr DO statement { $$ = new WhileNode($2, $4); }
		;

repeat : REPEAT stlist UNTIL expr { $$ = new RepeatNode($2, $4); }
		;

for     : FOR assign TO expr DO statement { $$ = new ForNode($2 as AssignNode, $4, $6); }
	    ;

write   : WRITE LEFT_PR expr RIGHT_PR { $$ = new WriteNode($3); }
		;

if 		: IF expr THEN statement { $$ = new IfNode($2, $4); } %prec IFX
		| IF expr THEN statement ELSE statement { $$ = new IfNode($2, $4, $6); }
		;

idList  : ident { $$ = new VarDefNode($1 as IdNode); }
		| idList COMMA ident { $1.Add($3 as IdNode); $$ = $1; }
		;

var     : VAR idList { $$ = $2; }
		;


%%

